require 'test/unit'
require 'rupat'
require 'fileutils'


class RupatTest < Test::Unit::TestCase

    # Test all editing features.
    def test_editing

        test_file = 'test/input/test_file1.txt'

        # Open file.
        r = Rupat.update( test_file ).useAlias

        # Should be at file beginning.
        assert_equal( r.l, 0 )

        # ------------------------------------------------------------
        # Test diff jumping commands and their return values:
        r.n;        assert_equal( r.l, 1 )
        r.p;        assert_equal( r.l, 0 )
        true;       assert_equal( r.n.l, 1 )
        true;       assert_equal( r.p.l, 0 )
        r.j( 5 );   assert_equal( r.l, 5 )
        true;       assert_equal( r.j.l, 0 )
        r.j1( 11 ); assert_equal( r.l, 10 )
        r.jl;       assert_equal( r.l, 9 )
        r.jl;       assert_equal( r.l, 9 )
        r.je;       assert_equal( r.l, 10 )


        # ------------------------------------------------------------
        # Test invalid lines:
        err = false
        begin
            r.j( 11 )
        rescue RupatMod::RupatInvalidLineError
            err = true
        end
        assert_equal( err, true )
        assert_equal( r.l, 10 )


        # ------------------------------------------------------------
        # Test excursion:
        l = r.excursion do
            r.j( 2 ).l
        end
        assert_equal( r.l, 10 )
        assert_equal( l, 2 )


        # ------------------------------------------------------------
        # Test searching:
        assert_equal( r.j(0).f( /Line 0/ ).l, 0 )
        assert_equal( r.f( /Line 1/ ).l, 1 )
        assert_equal( r.f( /ne 8/ ).l, 8 )
        l = r.excursion do
            r.f( /Line 9/ ).l
        end
        assert_equal( r.l, 8 )
        assert_equal( l, 9 )

        err = false
        begin
            r.findForward( /Line 1/ )
        rescue RupatMod::RupatSearchError
            err = true
        end
        assert_equal( err, true )

        err = false
        begin
            r.findBackward( /Line 1/ )
        rescue RupatMod::RupatSearchError
            err = true
        end
        assert_equal( err, false )
        assert_equal( r.l, 1 )

        err = false
        begin
            r.findBackward( /Line 9/ )
        rescue RupatMod::RupatSearchError
            err = true
        end
        assert_equal( err, true )
        assert_equal( r.l, 1 )

        b = r.findBlock( /Line 7/, /Line 8/ )
        assert_equal( b, [7,8] )
        assert_equal( r[ Range.new( *b ) ], ["Line 7", "Line 8"] )


        # ------------------------------------------------------------
        # Test content queries:
        assert_equal( r.j.g, "Line 0" )
        assert_equal( r.g( 9 ), "Line 9" )
        assert_equal( r.g( 10 ), nil )
        assert_equal( r.l, 0 )
        assert_equal( r.gm( 0, 1 ), ["Line 0", "Line 1"] )
        assert_equal( r.gm( 1 ), ["Line 0", "Line 1"] )
        assert_equal( r.cp( 1 ).copybuf, ["Line 0", "Line 1"] )


        # ------------------------------------------------------------
        # Test content manipulation:
        l = r.g
        r.s( "foobar" )
        assert_equal( r.g, "foobar" )
        r.s( l )
        assert_equal( r.length, 10 )


        r.replace do |c|
            c.gsub( /ne/, 'en' )
        end
        assert_equal( r.g, "Lien 0" )
        r.s( l )

        r.i( 'Line -1' )
        assert_equal( r.length, 11 )
        assert_equal( r.g, "Line -1" )
        r.d
        assert_equal( r.length, 10 )
        assert_equal( r.g, "Line 0" )

        c = ["foo", "bar" ]
        r.im( c )
        assert_equal( r.length, 12 )
        assert_equal( r.gm( 1 ), c )
        assert_equal( r.cp(1).pt.gm(1), c )
        assert_equal( r.length, 14 )
        assert_equal( r.dm(4).length, 10 )

        assert_equal( r.jl.a( 'foo' ).g, 'foo' )
        assert_equal( r.a( 'bar' ).g, 'bar' )
        assert_equal( r.length, 12 )
        assert_equal( r.gm(10,11), c )
        assert_equal( r.l, r.length-1 )
        assert_equal( r.am( "foo\nbar" ).length, 14 )
        assert_equal( r.gm(-1), c )
        assert_equal( r.ct( 10, 13 ).copybuf, c+c )

        len = r.length
        assert_equal( r.j.ifi( test_file ).length, 2*len )
        assert_equal( r.je.ifi( test_file ).length, 3*len )

        r.replaceAll( /Line/, 'foobar' )
        assert_equal( r.gm(5,6), [ 'foobar 5', 'foobar 6' ] )
        r.replaceWithin( /foobar/, 'Line', 10, 19 )


        # ------------------------------------------------------------
        # Test file saving:

        # Add lines to file and check for updates.
        r = Rupat.update( test_file ).useAlias
        ref = r.lines.dup
        r.ifi( test_file )
        r.save
        r = Rupat.open( test_file ).useAlias
        assert_equal( r.lines, ref+ref  )

        # File save with nil.
        r = Rupat.open( test_file ).useAlias
        r.ct( 10, r.last )
        err = false
        begin
            r.save
        rescue RupatMod::RupatTypeError
            err = true
        end
        assert_equal( err, true )

        # Save the file (with original content).
        r.save( test_file )

        # Edit and save to backup file.
        r = Rupat.edit( test_file ).useAlias
        bu = r.close
        assert_equal( File.exist?( bu ), true )
        FileUtils.rm_f( bu )

        # Create a new file with ".create".
        bu = test_file+'.dup'
        r = Rupat.create( test_file, bu ).useAlias
        r.close
        assert_equal( File.exist?( bu ), true )
        FileUtils.rm_f( bu )

        # Create file from lines.
        bu = test_file+'.dup'
        content = Rupat.open( test_file ).lines
        r = Rupat.lines( content )
        r.save( bu )
        assert_equal( File.exist?( bu ), true )
        assert_equal( r.length, 10 )
        FileUtils.rm_f( bu )

        # File-handle based concatenation.
        fh = File.open( bu, 'w' )
        r = Rupat.open( test_file )
        r.save( fh )
        r.save( fh )
        fh.close
        assert_equal( Rupat.open( bu ).length, 20 )
        FileUtils.rm_f( bu )

    end

end
